<?php

/*
 * Project: MinecraftLogFileReader
 * File name: online.php
 * Description: Reads the server logfile of Minecraft to provide useful information
 *
 * @original author Michael Csida, Copyright (C) 2011.
 * @auhtor Benjamin Kristensen
 * @version 1.7.4 (2014/01/11)
 * @see The GNU Public License (GPL)
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * Changelog:
 *
 * - v0.1.1 - 2011/06/16 - Upload of first working version (Michael Csida)
 * - v0.1.2 - 2011/06/17 - Minor bugfixes and source code layout changes (Benjamin Kristensen)
 * - v0.2.0 - 2011/06/17 - Added Total Play Time and a new layout (online users are on the top of the list and have an earth-background, offline users have a stone-background) (Michael Csida)
 * - v0.2.1 - 2011/06/18 - Minor bugfixes (Benjamin Kristensen)
 * - v0.3.0 - 2011/06/19 - Added option to display all users (not only users on whitelist), added debug-mode, layout changes (Michael Csida)
 *                         Added refresh rate to auto-reload page (Benjamin Kristensen)
 * - v0.3.1 - 2011/06/19 - Speed improvements (Benjamin Kristensen)
 * - v0.4.0 - 2011/08/18 - Added "Play Time per Day" (Michael Csida)
 * - v0.4.1 - 2011/09/02 - Script now ignores empty lines in the white-list. (Michael Csida)
 *                         Fixed a bug where "Play Time per Day" was greater than "Total Play Time" when the user's first login was within the last 24 hours. (Michael Csida)
 * - v0.5.0 - 2011/09/13 - Added an option to sort the list by Name, Total Play Time, Play Time per Day and Status. (Michael Csida)
 *                         Bugfix: Added "banning" and "kicking" to count as logoff/disconnect. Thanks to "TheRayden" from minecraftforum.net for reporting this and for his server logfile! (Michael Csida)
 *                         Date of first login will be displayed when hover over a user's name. (Michael Csida)
 *                         Added a new sky-image and text on top of the list. (Michael Csida)
 * - v0.6.0 - 2012/11/15 - Re-Design with CSS (Michael Csida)
 *                         Added support for "new" online-logfile-lines with IP address, old logfiles will not work with v0.6.0! (Michael Csida)
 *                         Added text field to change the value for removing offline users from the list after X days (Michael Csida)
 *                         Added User information to the top row (Michael Csida)
 * - v1.7.4 - 2014/01/11 - Added compatibility for Minecraft 1.7.4 (Michael Csida)
 */

/*-----------------------------------------------------------------------
--- VARIABLES YOU CAN / NEED TO CHANGE ----------------------------------
-----------------------------------------------------------------------*/

// Necessary:
$path_to_logs = '/home/minecraft/logs/';	// Absolute or relative path to the directory where the logfiles are located (don't forget the "/" at the end!)

// Optionally:
$use_whitelist = 0;							// Show users on whitelist ONLY? (will be faster) (1 = yes / 0 = no, show all users)
$path_to_whitelist = '/home/minecraft/';	// Absolute or relative path to the directory where the whitelist is located (don't forget the "/" at the end!)
$whitelist = 'white-list.txt'; 				// Name of your server whitelist file (will be ignored if $use_whitelist is 0)
$show_online_first = 1;						// Show online users first when sorting (this is the default setting which can be changed via a dropdown menu) (1 = Show online users first / 0 = Ignore online/offline status for sorting)
$sort_default = 1;							// Default column for sorting (1 = Name, 2 = Total Play Time, 3 = Play Time per Day, 4 = Status) (this is the default setting which can be changed by clicking on a column name)
$display_minute_after = 60; 				// After 60 seconds (default), minutes will be displayed instead of seconds (only for the "Status" column)
$display_hour_after = 7200; 				// After 7200 seconds (120 minutes) (default), hours will be displayd instead of minutes (only for the "Status" column)
$display_day_after = 172800; 				// After 172800 seconds (48 hours) (default), days will be displayed instead of hours (only for the "Status" column)
$remove_offline_after = 2592000;			// After 2592000 seconds (30 days) (default) offline, the users will not be shown in the list
$refresh_rate = 60;							// After 60 seconds (default), the page auto-reloads to update the values (set to 0 to disable auto-reload)
$debug = 0;									// To display debug information at the end of the page (1 = yes / 0 = no)

/*-----------------------------------------------------------------------
--- CODE (DO NOT CHANGE ANYTHING) ---------------------------------------
-----------------------------------------------------------------------*/

// Fill variables
$array_filenames = glob($path_to_logs.'*.log.gz');
$g = 0;
for($i=0; $i<count($array_filenames); $i++) {
	$array_gz_log = gzfile($array_filenames[$i]);
	$string_date = substr($array_filenames[$i], -19, 10);
	for($h=0; $h<count($array_gz_log); $h++) {
		$string_time = substr($array_gz_log[$h], 1, 8);
		$array_logfile[$g] = $string_date." ".$string_time." ".substr($array_gz_log[$h], 11);
		$g++;
	}
}
$string_date = date("Y-m-d", filemtime($path_to_logs.'latest.log'));
$array_latest_log = file($path_to_logs.'latest.log');
for($i=0; $i<count($array_latest_log); $i++) {
	$string_time = substr($array_latest_log[$i], 1, 8);
	$array_logfile[$g] = $string_date." ".$string_time." ".substr($array_latest_log[$i], 11);
	$g++;
}

date_default_timezone_set(@date_default_timezone_get());
$current_time = time();
$current_filename = basename($_SERVER['PHP_SELF']);
if ($use_whitelist == 1) {
	$file_whitelist = $path_to_whitelist.$whitelist;
	$array_whitelist = file($file_whitelist);
}
if ($_GET['input_remove_offline_after']) {
	$remove_offline_after = $_GET['input_remove_offline_after']*24*60*60;
}


// Sort - function for multidimensional arrays (from http://www.php.net/manual/en/function.sort.php#104464)
function sortmulti ($array, $index, $order, $natsort, $case_sensitive) {
	if(is_array($array) && count($array)>0) {
		foreach(array_keys($array) as $key)
			$temp[$key]=$array[$key][$index];
		if(!$natsort) {
			if ($order=='asc')
				asort($temp);
			else
				arsort($temp);
		} else {
			if ($case_sensitive==true)
				natsort($temp);
			else
				natcasesort($temp);
			if($order!='asc')
				$temp=array_reverse($temp,TRUE);
		}
		foreach(array_keys($temp) as $key)
			if (is_numeric($key))
				$sorted[]=$array[$key];
			else
				$sorted[$key]=$array[$key];
		return $sorted;
	}
	return $sorted;
}


// To calculate total runtime of the script
function getmicrotime(){
	list($usec, $sec) = explode(" ",microtime());
	return ((float)$usec + (float)$sec);
}

if ($debug == 1) {
	$time_start = getmicrotime();
}


// Temporary LogFile Array with Online/Offline - Info rows only
if ($debug == 1) { // Step1 Start
	$time_step1_start = getmicrotime();
}
$h = 0;
foreach($array_logfile as $log_line) {
	if(preg_match('@lost connection:|logged in with entity id@', $log_line)) {
		$array_logfile_temp_onoff[$h] = $log_line;
		$h++;
	}
}
if ($debug == 1) { // Step1 End
	$time_step1_end = getmicrotime();
	$time_step1 = round($time_step1_end - $time_step1_start,4);
}


// Create array for username, current userstatus, last change of online/offline status, total play time, play time per day and first login
if ($debug == 1) { // Step2 Start
	$time_step2_start = getmicrotime();
}
$h = 0;
if ($use_whitelist == 1) {
	for ($i = 0; $i < count($array_whitelist); $i++) {
		if (rtrim(ltrim($array_whitelist[$i])) != "") {
			$array_users[$h]['name'] = rtrim(ltrim($array_whitelist[$i]));
			$array_users[$h]['status'] = 'UNCLEAR';
			$array_users[$h]['lastchange'] = 0;
			$array_users[$h]['totaltime'] = 0;
			$array_users[$h]['hoursperday'] = 0;
			$array_users[$h]['firstlogin'] = 0;
			$h++;
		}		
	}
} else {
	$array_users_temp = array();
	$h = 0;
	for ($i = 0; $i < count($array_logfile_temp_onoff); $i++) {
		if (preg_match('#logged in with entity id#', $array_logfile_temp_onoff[$i])) {
			$array_help1 = explode(" [Server thread/INFO]: ", $array_logfile_temp_onoff[$i]);
			$array_help2 = explode("[", $array_help1[1], 2);
			$flipped_array = array_flip($array_users_temp); 
			if (!isset($flipped_array[$array_help2[0]])) { 
				$array_users_temp[$h] = $array_help2[0];
				$h++; 
			}
		}
	}
	for ($i = 0; $i < count($array_users_temp); $i++) {
		$array_users[$i]['name'] = $array_users_temp[$i];
		$array_users[$i]['status'] = 'UNCLEAR';
		$array_users[$i]['lastchange'] = 0;
		$array_users[$i]['totaltime'] = 0;
		$array_users[$i]['hoursperday'] = 0;
		$array_users[$i]['firstlogin'] = 0;
	}
}
$count_user = count($array_users);
if ($debug == 1) { // Step2 End
	$time_step2_end = getmicrotime();
	$time_step2 = round($time_step2_end - $time_step2_start,4);
}


// Script: Finding out Online/Offline - Status and set timestamp of last change
if ($debug == 1) { // Step3 Start
	$time_step3_start = getmicrotime();
}
$users_checked = 0;
$lines_to_check = count($array_logfile_temp_onoff) - 1;

while (($users_checked < $count_user) && ($lines_to_check >= 0)) {
	$array_temp1 = explode(" [Server thread/INFO]: ", $array_logfile_temp_onoff[$lines_to_check]);
	if (preg_match('#logged in with entity id#', $array_temp1[1])) {
		$array_temp2 = explode("[", $array_temp1[1], 2);
	} else {
		$array_temp2 = explode(" ", $array_temp1[1], 2);
	}
	$user_to_check = $array_temp2[0];
	for ($i = 0; $i < $count_user; $i++) {
		if ((strtoupper($array_users[$i]['name']) == strtoupper($user_to_check)) && $array_users[$i]['status'] == 'UNCLEAR') {
			if (preg_match('#lost connection:#', $array_temp2[1])) $array_users[$i]['status'] = 'OFFLINE';
			if (preg_match('#logged in with entity id#', $array_temp2[1])) $array_users[$i]['status'] = 'ONLINE';
			$array_temp3 = explode(" ", $array_temp1[0]);
			$array_temp4 = explode("-", $array_temp3[0]);
			$array_temp5 = explode(":", $array_temp3[1]);
			$array_users[$i]['lastchange'] = mktime($array_temp5[0],$array_temp5[1],$array_temp5[2],$array_temp4[1],$array_temp4[2],$array_temp4[0]);
			$array_users[$i]['name'] = $user_to_check;
			$users_checked++;
		}
	}
	$lines_to_check--;
}
if ($debug == 1) { // Step3 End
	$time_step3_end = getmicrotime();
	$time_step3 = round($time_step3_end - $time_step3_start,4);
}


// Finding out Total Play Time
if ($debug == 1) { // Step4 Start
	$time_step4_start = getmicrotime();
}

$playtime_all_users = 0;
$array_logline_timestamp = array();
$array_logline_user = array();
$array_logline_status = array();

for ($i = 0; $i < $count_user; $i++) { // Check one user after another
	$f = 0;
	unset($array_logfile_temp_user);
	for ($g = 0; $g < count($array_logfile_temp_onoff); $g++) { // Create array for online/offline information for this user only
		$array_temp6 = explode(" [Server thread/INFO]: ", $array_logfile_temp_onoff[$g]);
		if (preg_match('#logged in with entity id#', $array_temp6[1])) {
			$array_temp7 = explode("[", $array_temp6[1], 2);
		} else {
			$array_temp7 = explode(" ", $array_temp6[1], 2);
		}
		$user_to_check = $array_temp7[0];
		if (strtoupper($array_users[$i]['name']) == strtoupper($user_to_check)) {
			$array_logfile_temp_user[$f] = $array_logfile_temp_onoff[$g];
			$f++;
		}
	}
	for ($e = 0; $e < count($array_logfile_temp_user); $e++) { // Store all the loglines of the user-array to other arrays once
		$array_logline_temp1 = explode(" [Server thread/INFO]: ", $array_logfile_temp_user[$e]);
		$array_logline_temp2 = explode(" ", $array_logline_temp1[0]);
		$array_logline_temp3 = explode("-", $array_logline_temp2[0]);
		$array_logline_temp4 = explode(":", $array_logline_temp2[1]);
		$array_logline_temp5 = explode(" ", $array_logline_temp1[1], 2);

		$array_logline_timestamp[$e] = mktime($array_logline_temp4[0],$array_logline_temp4[1],$array_logline_temp4[2],$array_logline_temp3[1],$array_logline_temp3[2],$array_logline_temp3[0]);
		
		if (preg_match('#logged in with entity id#', $array_logline_temp5[1])) {
			$array_logline_status[$e] = 'ONLINE';
			$array_logline_user[$e] = $array_logline_temp5[0];
			}
		if (preg_match('#lost connection:#', $array_logline_temp5[1])) {
			$array_logline_status[$e] = 'OFFLINE';
			$array_logline_user[$e] = $array_logline_temp5[0];
		}
	}
	for ($h = 0; $h < count($array_logfile_temp_user); $h++) { // Calculating Total Played Time
		if ($h == 0) { // First entry of the user logfile
			$temp_current_timestamp = $array_logline_timestamp[$h];
			$array_users[$i]['firstlogin'] = $array_logline_timestamp[$h]; // For "hours per day"
		} else if (($h > 0) && ($h < count($array_logfile_temp_user)-1)){
			if (($array_logline_status[$h] == 'ONLINE') && ($array_logline_status[$h-1] == 'OFFLINE')) {
				$temp_current_timestamp = $array_logline_timestamp[$h];
			}
			if (($array_logline_status[$h] == 'OFFLINE') && ($array_logline_status[$h-1] == 'ONLINE')) {
				$diff = $array_logline_timestamp[$h] - $temp_current_timestamp;
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
		} else if ($h == count($array_logfile_temp_user)-1) { // Last entry of the user logfile
			if ($array_logline_status[$h] == 'ONLINE') {
				$diff = $current_time - $array_logline_timestamp[$h];
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
			if ($array_logline_status[$h] == 'OFFLINE') {
				$diff = $array_logline_timestamp[$h] - $temp_current_timestamp;
				$array_users[$i]['totaltime'] = $array_users[$i]['totaltime'] + $diff;
			}
		}
	}
	$playtime_all_users = $array_users[$i]['totaltime'] + $playtime_all_users;
}
if ($debug == 1) { // Step4 End
	$time_step4_end = getmicrotime();
	$time_step4 = round($time_step4_end - $time_step4_start,4);
}


// Calculating hours per day
if ($debug == 1) { // Step5 Start
	$time_step5_start = getmicrotime();
}
for ($i = 0; $i < $count_user; $i++) {
	$active_days = ($current_time - $array_users[$i]['firstlogin']) / 60 / 60 / 24;
	$array_users[$i]['hoursperday'] = round(($array_users[$i]['totaltime'] /60 / 60 / $active_days), 3);
}
if ($debug == 1) { // Step5 End
	$time_step5_end = getmicrotime();
	$time_step5 = round($time_step5_end - $time_step5_start,4);
}


// Sort user array
if ($debug == 1) { // Step6 Start
	$time_step6_start = getmicrotime();
}
if (!isset($_GET['sort_by'])) {
	switch($sort_default) {
		case 1: $_GET['sort_by'] = 'name'; $_GET['sort_order'] = 'asc'; break;
		case 2: $_GET['sort_by'] = 'totaltime'; $_GET['sort_order'] = 'desc'; break;
		case 3: $_GET['sort_by'] = 'hoursperday'; $_GET['sort_order'] = 'desc'; break;
		case 4: $_GET['sort_by'] = 'lastchange'; $_GET['sort_order'] = 'desc'; break;
	}
}

if (!isset($_GET['show_online_first'])) {
	$_GET['show_online_first'] = $show_online_first;
}

if ($_GET['show_online_first'] == 1) { // Online users first
	if ($_GET['sort_by'] == 'name') {
		$temp_array_users_sorted = sortmulti($array_users, $_GET['sort_by'], $_GET['sort_order'], TRUE, FALSE);
	} else {
		$temp_array_users_sorted = sortmulti($array_users, $_GET['sort_by'], $_GET['sort_order'], FALSE, FALSE);	
	}
	$h=0;
	for ($i = 0; $i < $count_user; $i++) {
		if ($temp_array_users_sorted[$i]['status'] == 'ONLINE') {
			$array_users_sorted[$h]['name'] = $temp_array_users_sorted[$i]['name'];
			$array_users_sorted[$h]['status'] = $temp_array_users_sorted[$i]['status'];
			$array_users_sorted[$h]['lastchange'] = $temp_array_users_sorted[$i]['lastchange'];
			$array_users_sorted[$h]['totaltime'] = $temp_array_users_sorted[$i]['totaltime'];
			$array_users_sorted[$h]['hoursperday'] = $temp_array_users_sorted[$i]['hoursperday'];
			$array_users_sorted[$h]['firstlogin'] = $temp_array_users_sorted[$i]['firstlogin'];
			$h++;
		}
	}
	for ($i = 0; $i < $count_user; $i++) {
		if ($temp_array_users_sorted[$i]['status'] != 'ONLINE') {
			$array_users_sorted[$h]['name'] = $temp_array_users_sorted[$i]['name'];
			$array_users_sorted[$h]['status'] = $temp_array_users_sorted[$i]['status'];
			$array_users_sorted[$h]['lastchange'] = $temp_array_users_sorted[$i]['lastchange'];
			$array_users_sorted[$h]['totaltime'] = $temp_array_users_sorted[$i]['totaltime'];
			$array_users_sorted[$h]['hoursperday'] = $temp_array_users_sorted[$i]['hoursperday'];
			$array_users_sorted[$h]['firstlogin'] = $temp_array_users_sorted[$i]['firstlogin'];
			$h++;
		}
	}
} else { // Ignore online/offline status for sorting
	if ($_GET['sort_by'] == 'name') {
		$array_users_sorted = sortmulti($array_users, $_GET['sort_by'], $_GET['sort_order'], TRUE, FALSE);
	} else {
		$array_users_sorted = sortmulti($array_users, $_GET['sort_by'], $_GET['sort_order'], FALSE, FALSE);
	}
}
if ($debug == 1) { // Step6 End
	$time_step6_end = getmicrotime();
	$time_step6 = round($time_step6_end - $time_step6_start,4);
}

// Calculating (online) users in the list
$count_user_displayed = 0;
$count_user_online = 0;
for ($i = 0; $i < $count_user; $i++) {
	if (($current_time - $array_users_sorted[$i]['lastchange'] < $remove_offline_after) || ($array_users_sorted[$i]['status'] == 'ONLINE')) {
		$count_user_displayed++;
		if ($array_users_sorted[$i]['status'] == 'ONLINE') {
			$count_user_online++;
		}
	}
}
?>

<!-----------------------------------------------------------------------
--- OUTPUT --------------------------------------------------------------
------------------------------------------------------------------------>

<html>

<head>
	<link rel=stylesheet type="text/css" href="style.css">
	<?php if ($refresh_rate > 0) echo "<meta http-equiv=\"refresh\" content=\"".$refresh_rate."\">\n"; ?>
	<script type="text/javascript">
		function checkForm() {
			if (isNaN(document.form.input_remove_offline_after.value) || document.form.input_remove_offline_after.value > 999 || document.form.input_remove_offline_after.value < 0.1) {
				alert ("Please use a valid number\nRange: 0.1 to 999\nUse a dot as decimal point.");
				document.form.input_remove_offline_after.focus();
				return false;
			}
		}
	</script>
</head>

<body>

<table class="main">
	<tr class="toprow">
		<td colspan=5><div class="shadow">MinecraftLogFileReader</div><div class="shadow_small">v1.7.4</div><div class="shadow_middle">Total Play Time of all Users: <?php echo sprintf("%01.2f", round($playtime_all_users/60/60/24, 2)); ?> days<br>Total Users in <?php if($use_whitelist == 1) { echo "whitelist"; } else { echo "logfile"; } ?>: <?php echo $count_user; ?><br>Users displayed: <?php echo $count_user_displayed; ?><br>Users online: <?php echo $count_user_online; ?></div><div class="pic">&nbsp;</div></td>
	</tr>
	<tr class="headrow">
		<td>&nbsp;</td>
		<td><a class="togreen" href="<?php echo $current_filename; ?>?sort_by=name&sort_order=asc&show_online_first=<?php echo $_GET['show_online_first']; if($_GET['input_remove_offline_after']) { echo "&input_remove_offline_after=".$_GET['input_remove_offline_after']; } ?>">Name</a></td>
		<td><a class="togreen" href="<?php echo $current_filename; ?>?sort_by=totaltime&sort_order=desc&show_online_first=<?php echo $_GET['show_online_first']; if($_GET['input_remove_offline_after']) { echo "&input_remove_offline_after=".$_GET['input_remove_offline_after']; } ?>">Total Play Time</a></td>
		<td><a class="togreen" href="<?php echo $current_filename; ?>?sort_by=hoursperday&sort_order=desc&show_online_first=<?php echo $_GET['show_online_first']; if($_GET['input_remove_offline_after']) { echo "&input_remove_offline_after=".$_GET['input_remove_offline_after']; } ?>">Play Time per Day</a></td>
		<td><a class="togreen" href="<?php echo $current_filename; ?>?sort_by=lastchange&sort_order=desc&show_online_first=<?php echo $_GET['show_online_first']; if($_GET['input_remove_offline_after']) { echo "&input_remove_offline_after=".$_GET['input_remove_offline_after']; } ?>">Status</a></td>
	</tr>
	<?php
	for ($i = 0; $i < $count_user; $i++) {
		if (($current_time - $array_users_sorted[$i]['lastchange'] < $remove_offline_after) || ($array_users_sorted[$i]['status'] == 'ONLINE')) {
			if ($array_users_sorted[$i]['status'] == 'ONLINE') {
				echo "<tr class=\"userrow_online\">";
			} else {
				echo "<tr class=\"userrow_offline\">";
			}
			echo "\n<td class=\"usercell\"><img src=\"pictures/";
			if ($array_users_sorted[$i]['status'] == 'ONLINE') echo "online";
			else if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo "offline";
			else echo "unclear";
			echo ".jpg\"></td>\n";
			$temp_totaltime_minutes = sprintf("%01.1f", round($array_users_sorted[$i]['totaltime']/60, 1));
			$temp_totaltime_hours = sprintf("%01.1f", round($array_users_sorted[$i]['totaltime']/60/60, 1));
			$temp_totaltime_days = sprintf("%01.2f", round($array_users_sorted[$i]['totaltime']/60/60/24, 2));
			$temp_firstlogin = date("Y-m-d",$array_users_sorted[$i]['firstlogin']);
			echo "<td class=\"usercell\"><div class=\"ToolText\" onMouseOver=\"javascript:this.className='ToolTextHover'\" onMouseOut=\"javascript:this.className='ToolText'\">".$array_users_sorted[$i]['name']."<span>First Login: ".$temp_firstlogin."</span></div></td>\n";
			echo "<td class=\"usercell\"><span class=\"vnn10white\">".$temp_totaltime_hours." hours (".$temp_totaltime_days." days)</td>";
			$output_hours = sprintf("%01.2f", round($array_users_sorted[$i]['hoursperday'], 2));
			$output_minutes = sprintf("%01.1f", round(($array_users_sorted[$i]['hoursperday'] * 60), 1));
			if ($output_minutes > $temp_totaltime_minutes) {
				echo "<td class=\"usercell\"><span class=\"vnn10white\">".$temp_totaltime_minutes." minutes (".$temp_totaltime_hours." hours)</span></td>\n";
			} else {
				echo "<td class=\"usercell\"><span class=\"vnn10white\">".$output_minutes." minutes (".$output_hours." hours)</span></td>\n";
			}
			echo "<td class=\"usercell_status\"><span class=\"vnn10white\">";
			if ($array_users_sorted[$i]['status'] == 'ONLINE') echo "Online for ";
			else if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo "Last online: ";
			else echo "Unclear";
			$diff = $current_time - $array_users_sorted[$i]['lastchange'];
			if ($array_users_sorted[$i]['lastchange'] == 0) {
				echo "";
			} else if ($diff >= $display_day_after) {
				$temp = floor($diff/86400);
				echo $temp." day"; if ($temp > 1) echo "s";
			} else if ($diff >= $display_hour_after) {
				$temp = floor($diff/3600);
				echo $temp." hour"; if ($temp > 1) echo "s";
			} else if ($diff >= $display_minute_after) {
				$temp = floor($diff/60);
				echo $temp." minute"; if ($temp > 1) echo "s";
			} else {
				echo $diff." second"; if ($diff > 1) echo "s";
			}
			if ($array_users_sorted[$i]['status'] == 'OFFLINE') echo " ago";		
			echo "</span></td>\n";
			echo "</tr>\n";
		}
	}
	echo "<tr class=\"bottomrow\"><td class=\"bottomcell\" colspan=5>";
	echo "<form name=\"form\" action=\"".$current_filename."\" method=\"GET\" onSubmit=\"return checkForm()\" style=\"display:inline;\">";
	echo "<span class=\"vnn10white\">When sorting: </span>\n";
	echo "<input type=\"hidden\" name=\"sort_by\" value=\"".$_GET['sort_by']."\">\n";
	echo "<input type=\"hidden\" name=\"sort_order\" value=\"".$_GET['sort_order']."\">\n";
	echo "<select name=\"show_online_first\" onChange=\"submit()\">\n";
	echo "<option value=1";	if ($_GET['show_online_first'] == 1) echo " selected"; echo ">Show online users first</option>\n";
	echo "<option value=0";	if ($_GET['show_online_first'] == 0) echo " selected"; echo ">Ignore online/offline status</option>\n";
	echo "</select>\n";
	echo "<span class=\"vnn10white\"> (List is currently sorted by \"";
	switch ($_GET['sort_by']) {
		case 'name': echo "Name"; break;
		case 'totaltime': echo "Total Play Time"; break;
		case 'hoursperday': echo "Play Time per Day"; break;
		case 'lastchange': echo "Status (last change)"; break;
	}
	echo "\")<br>\n";
	echo "<span class=\"vnn10white\">Hide users who are offline for more than </span><input type=\"text\" name=\"input_remove_offline_after\" value=\"".sprintf("%01.2f", round($remove_offline_after/60/60/24, 2))."\" size=\"5\" maxlength=\"4\" style=\"background-color:#666666; color:#FFFFFF; font-family:Verdana,Arial,Helvetica; border:2px solid #333333; text-align:center;\"><span class=\"vnn10white\"> days</span>";
	echo "</form></td></tr>\n";
	if ($debug == 1) {
		$time_end = getmicrotime();
		$time = round($time_end - $time_start,4);
		$lines_read = count($array_logfile_temp_onoff) - $lines_to_check - 1;
		$remove_offline_days = sprintf("%01.2f", round($remove_offline_after/60/60/24, 2));
		echo "<tr class=\"bottomrow\">";
		echo "<td class=\"debugcell\" colspan=5><span class=\"vnn10white\">Page was generated in ".$time." seconds.<br>";
		echo "(Step1: ".$time_step1."s / Step2: ".$time_step2."s / Step3: ".$time_step3."s / Step4: ".$time_step4."s / Step5: ".$time_step5."s / Step6: ".$time_step6."s)<br>";
		echo "Users found in "; if ($use_whitelist == 1) echo "whitelist"; else echo "logfile"; echo ": ".$count_user."<br>";
		echo "Users displayed: ".$count_user_displayed." (showing only users who are not offline for more than ".sprintf("%01.2f", round($remove_offline_after/60/60/24, 2))." days)<br>";
		echo "Total logfile lines: ".count($array_logfile)."<br>";
		echo "Logfile lines with online/offline information: ".count($array_logfile_temp_onoff)."<br>";
		echo "Logfile lines read to find out online/offline status: ".$lines_read."</span></td>";
		echo "</tr>";
	}
	?>
</table>

<br><br>

</body>

</html>